/************************************************************************
 *                                                                      *
 *  dialog.c  --  xcontrol X-dialog treating.                           *
 *                                                                      *
 *  This file is part of the control/rt_control/xcontrol package which  *
 *  is free software; you can redistribute it and/or modify it under    *
 *  the terms of the version 2 of GNU General Public License as         *
 *  published by the Free Software Foundation.                          *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft 1999                                                       *
 *  Peter Wurmsdobler                                                   *
 *                                                                      *
 ************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "common.h"
#include "dialog.h"

/*
 *	Debug macros for dialogs
 */
//#define DIALOG_DEBUG
#ifdef DIALOG_DEBUG
	#define DPRINTF0(dialog)		printf(dialog)
	#define DPRINTF1(dialog,a)		printf(dialog,a)
	#define DPRINTF2(dialog,a,b)	printf(dialog,a,b)
#else
	#define DPRINTF0(dialog)
	#define DPRINTF1(dialog,a)
	#define DPRINTF2(dialog,a,b)
#endif

/*
 *	definition of widgets for dialog window
 */
unsigned int error_length;
GtkWidget *Dialog_Error_Window;
GtkWidget *Dialog_Error_Window_Container;
GtkWidget *Dialog_Error_Window_Text;
GtkWidget *Dialog_Error_Window_Ok_Button;

/*
 *	definition of widgets for dialog window
 */
unsigned int info_length;
GtkWidget *Dialog_Info_Window;
GtkWidget *Dialog_Info_Window_Container;
GtkWidget *Dialog_Info_Window_Text;
GtkWidget *Dialog_Info_Window_Ok_Button;


/************************************************************************
 *  Error dialog                                                        *
 ************************************************************************/

/*
 *	Callback function of error dialog window OK button
 */
void dialog_error_window_Ok_Button_activate( void )
	{
	gtk_widget_hide( Dialog_Error_Window );
	gtk_text_backward_delete( GTK_TEXT (Dialog_Error_Window_Text), error_length );
	error_length = 0;
	}


/*
 *	definition of dialog window
 */
void dialog_error_window_create( void ) 
	{
	Dialog_Error_Window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_set_usize( Dialog_Error_Window, 200, 250 );
	gtk_window_set_title( GTK_WINDOW (Dialog_Error_Window), 
    	DIALOG_ERROR_WINDOW_TEXT );
	gtk_window_set_policy( GTK_WINDOW (Dialog_Error_Window), 
    	FALSE, FALSE, FALSE );
	
	Dialog_Error_Window_Container = gtk_fixed_new();
  	gtk_container_add( GTK_CONTAINER (Dialog_Error_Window), 
		Dialog_Error_Window_Container );

	Dialog_Error_Window_Text = gtk_text_new( NULL, NULL );
    gtk_fixed_put( GTK_FIXED(Dialog_Error_Window_Container), 
		Dialog_Error_Window_Text, 25, 25);
	error_length = 0;
                              				
	Dialog_Error_Window_Ok_Button = 
    	gtk_button_new_with_label( OK_BUTTON_TEXT );
	gtk_fixed_put (GTK_FIXED( Dialog_Error_Window_Container ), 
		Dialog_Error_Window_Ok_Button, 80, 200 );
	gtk_widget_set_usize( Dialog_Error_Window_Ok_Button, 50, 25 );
	gtk_signal_connect( GTK_OBJECT (Dialog_Error_Window_Ok_Button), "clicked",
		GTK_SIGNAL_FUNC ( dialog_error_window_Ok_Button_activate ), NULL );
	}


/*
 *	pop up dialog window with text
 */
void dialog_error_window( char *error_text, int nbytes ) 
	{
	if ( Dialog_Error_Window == NULL )
		dialog_error_window_create();
	
    gtk_text_insert (GTK_TEXT (Dialog_Error_Window_Text), NULL, NULL, NULL, 
    	error_text, nbytes );	
    gtk_text_insert (GTK_TEXT (Dialog_Error_Window_Text), NULL, NULL, NULL, 
    	"\n", 1 );	
    error_length += nbytes + 1;
        
	gtk_widget_show_all( Dialog_Error_Window );
	}


/************************************************************************
 *  Info dialog                                                         *
 ************************************************************************/

/*
 *	Callback function of error dialog window OK button
 */
void dialog_info_window_Ok_Button_activate( void )
	{
	gtk_widget_hide( Dialog_Info_Window );
	gtk_text_backward_delete( GTK_TEXT (Dialog_Info_Window_Text), info_length );
	info_length = 0;
	}


/*
 *	definition of info dialog window
 */
void dialog_info_window_create( void ) 
	{
	Dialog_Info_Window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_set_usize( Dialog_Info_Window, 400, 150 );
	gtk_window_set_title( GTK_WINDOW (Dialog_Info_Window), 
    	DIALOG_INFO_WINDOW_TEXT );
	gtk_window_set_policy( GTK_WINDOW (Dialog_Info_Window), 
    	FALSE, FALSE, FALSE );
	
	Dialog_Info_Window_Container = gtk_fixed_new();
  	gtk_container_add( GTK_CONTAINER (Dialog_Info_Window), 
		Dialog_Info_Window_Container );

	Dialog_Info_Window_Text = gtk_text_new( NULL, NULL );
    gtk_fixed_put( GTK_FIXED(Dialog_Info_Window_Container), 
		Dialog_Info_Window_Text, 25, 25);
	info_length = 0;
				
	Dialog_Info_Window_Ok_Button = 
    	gtk_button_new_with_label( OK_BUTTON_TEXT );
	gtk_fixed_put (GTK_FIXED( Dialog_Info_Window_Container ), 
		Dialog_Info_Window_Ok_Button, 175, 90 );
	gtk_widget_set_usize( Dialog_Info_Window_Ok_Button, 50, 25 );
	gtk_signal_connect( GTK_OBJECT (Dialog_Info_Window_Ok_Button), "clicked",
		GTK_SIGNAL_FUNC ( dialog_info_window_Ok_Button_activate ), NULL );
	}


/*
 *	pop up dialog window with text
 */
void dialog_info_window( char *info_text, int nbytes ) 
	{
	if ( Dialog_Info_Window == NULL )
		dialog_info_window_create();

    gtk_text_insert (GTK_TEXT (Dialog_Info_Window_Text), NULL, NULL, NULL, 
    	info_text, nbytes );	
    gtk_text_insert (GTK_TEXT (Dialog_Info_Window_Text), NULL, NULL, NULL, 
    	"\n", 1 );	       
	info_length += nbytes + 1;

	gtk_widget_show_all( Dialog_Info_Window );
	}
