/************************************************************************
 *                                                                      *
 *  measure.c  --  This file draw the graph which representing          *
 *  the measure.                                                        *
 *                                                                      *
 *  This file is part of the control/rt_control/xcontrol package which  *
 *  is free software; you can redistribute it and/or modify it under    *
 *  the terms of the version 2 of GNU General Public License as         *
 *  published by the Free Software Foundation.                          *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft 1999                                                       *
 *  Peter Wurmsdobler                                                   *
 *                                                                      *
 ************************************************************************/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gtkplot.h>
#include <gtkplotlayout.h>
#include "common.h"
#include "misc.h"
#include "control.h"
#include "xcontrol.h"
#include "measure.h"
#include "print.h"

/*
 *	Debug macros for 
 */
//#define MEASURE_DEBUG
#ifdef MEASURE_DEBUG
	#define DPRINTF0(message)		printf(message)
	#define DPRINTF1(message,a)		printf(message,a)
	#define DPRINTF2(message,a,b)	printf(message,a,b)
#else
	#define DPRINTF0(message)
	#define DPRINTF1(message,a)
	#define DPRINTF2(message,a,b)
#endif


/************************************************************************
 *  definition of widgets for measure                                   *
 ************************************************************************/


/*
 *	definition of widgets for force measure window
 */
GtkWidget *Measure_Quit_Menu;
GtkWidget *Measure_Print_Menu;
GtkWidget *Measure_Export_Menu;
GtkWidget *Measure_File; 
GtkWidget *Measure_Menu_File; 
GtkWidget *Measure_Menu;
GtkWidget *Measure_Window;
GtkWidget *Measure_Container;
GtkWidget *Measure_Vbox;
GtkWidget *Measure_Scrollw;
GtkWidget *Measure_Layout;
GtkWidget *Measure_Graph;


/* 
 *	attributes of plots 
 */
GtkPlotData *Measure_Y_Dataset;
GtkPlotData *Measure_U_Dataset;

/*
 *	vector of points to plot
 */
GtkPlotPoint y[SAMPLES];
GtkPlotPoint u[SAMPLES];

GdkColor point_color;
GdkColor y_color;
GdkColor u_color;
GdkColor measure_color;


void measure_window_resize( void )
	{
	gtk_widget_set_usize( GTK_WIDGET( Measure_Graph ),
		( GTK_WIDGET( Measure_Window )->allocation.width ),
		( GTK_WIDGET( Measure_Window )->allocation.height ) );
	}


void measure_menu_print_activate( void )
	{
	print_dialog_window( GTK_PLOT( Measure_Graph ) );
	}



void measure_window_close( void )
	{
	gtk_widget_hide( Measure_Window );
    }


/************************************************************************
 *  definition of measure window drawing function                       *
 ************************************************************************/

void measure_window_create( void )
	{
	Measure_Window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_set_usize( Measure_Window, 480, 380 );
	gtk_window_set_title( GTK_WINDOW( Measure_Window ), 
		MEASURE_WINDOW_TEXT );
	gtk_window_set_policy( GTK_WINDOW( Measure_Window ), 
		TRUE, TRUE, FALSE );

	gtk_signal_connect( GTK_OBJECT( Measure_Window ), 
		"size_allocate",
		GTK_SIGNAL_FUNC( measure_window_resize ),
		NULL );
	
	Measure_Container = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER( Measure_Window ),
		Measure_Container );

	Measure_Menu = gtk_menu_bar_new();
  	gtk_fixed_put( GTK_FIXED( Measure_Container ),
  		Measure_Menu, 0,0 );
  	gtk_widget_set_usize( Measure_Menu, 1200, 30 );

  	Measure_File = gtk_menu_item_new_with_label 
  		( FILE_TEXT );
  	gtk_container_add( GTK_CONTAINER( Measure_Menu ), 
  		Measure_File );
  	
  	Measure_Menu_File = gtk_menu_new();
  	gtk_menu_item_set_submenu( GTK_MENU_ITEM( Measure_File ),
  		Measure_Menu_File );
  	 	
  	Measure_Print_Menu = gtk_menu_item_new_with_label 
  		( PRINT_TEXT );
  	gtk_container_add( GTK_CONTAINER( Measure_Menu_File ),
  		Measure_Print_Menu );
  	gtk_signal_connect( GTK_OBJECT( Measure_Print_Menu ), 
  		"activate",
  		GTK_SIGNAL_FUNC( measure_menu_print_activate ),
  		NULL );
      	
  	Measure_Quit_Menu = gtk_menu_item_new_with_label 
  		( QUIT_TEXT );
  	gtk_container_add( GTK_CONTAINER( Measure_Menu_File ),
  		Measure_Quit_Menu );
  	gtk_signal_connect( GTK_OBJECT( Measure_Quit_Menu ), 
  		"activate",
  		GTK_SIGNAL_FUNC( measure_window_close ),
  		NULL );
    
  	Measure_Vbox = gtk_vbox_new( FALSE,0 );
  	gtk_fixed_put( GTK_FIXED( Measure_Container ),
  		Measure_Vbox, 0, 30 );
 	gtk_widget_set_usize( Measure_Vbox, 1200, 1000 );

 	Measure_Scrollw = gtk_scrolled_window_new( NULL, NULL );
 	gtk_container_border_width( GTK_CONTAINER 
 		( Measure_Scrollw ), 0 );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW
 		( Measure_Scrollw ), 
 		GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC );
 	gtk_box_pack_start( GTK_BOX( Measure_Vbox ),
 		Measure_Scrollw, TRUE, TRUE, 0 );
	
	Measure_Layout = gtk_plot_layout_new( 1200, 1000 );
 	gtk_container_add( GTK_CONTAINER( Measure_Scrollw ),
 		Measure_Layout );
 		
	Measure_Graph = gtk_plot_new( NULL );
	gtk_plot_move_resize( GTK_PLOT( Measure_Graph ),
    	0.12, 0.09, 0.71, 0.7 );
	gtk_plot_legends_move( GTK_PLOT( Measure_Graph ),
    	0.76, 0.01 );
	gtk_plot_axis_hide_title( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_TOP );
 	gtk_plot_axis_hide_title( GTK_PLOT( Measure_Graph ),
 		GTK_PLOT_AXIS_RIGHT );
	gtk_plot_axis_set_title( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_BOTTOM, MEASURE_TIME_TEXT );
	gtk_plot_axis_move_title( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_LEFT, 0, 0.0, 0.35 );
	gtk_plot_axis_move_title( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_BOTTOM, 0, 0.46, 0.85 );
	gtk_plot_axis_set_title( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_LEFT, MEASURE_SIGNAL_TEXT );
	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ), 
		GTK_PLOT_SHOW_TOP_AXIS );
 	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ), 
 		GTK_PLOT_SHOW_RIGHT_AXIS );
	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ), 
		GTK_PLOT_SHOW_X0 );
	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ), 
		GTK_PLOT_SHOW_Y0 );
	gtk_plot_layout_add_plot( GTK_PLOT_LAYOUT( Measure_Layout ),
		GTK_PLOT( Measure_Graph ), 25,5 );
	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ),GTK_PLOT_SHOW_V_GRID );
 	GTK_PLOT_SET_FLAGS( GTK_PLOT( Measure_Graph ),GTK_PLOT_SHOW_H_GRID );
	
	Measure_Y_Dataset = gtk_plot_dataset_new 
		( GTK_PLOT( Measure_Graph) );
	gtk_plot_add_dataset( GTK_PLOT( Measure_Graph ),
		Measure_Y_Dataset );
	gtk_plot_dataset_set_line_attributes( Measure_Y_Dataset,
		GTK_PLOT_LINE_SOLID, 2, y_color );
	gtk_plot_dataset_set_connector( Measure_Y_Dataset,
		GTK_PLOT_CONNECT_HV_STEP );
	gtk_plot_dataset_set_symbol( Measure_Y_Dataset,
    	GTK_PLOT_SYMBOL_CIRCLE,
    	GTK_PLOT_SYMBOL_OPAQUE,
    	4, 1, point_color );
    gtk_plot_dataset_set_legend( Measure_Y_Dataset,
        MEASURE_Y_TEXT );

	Measure_U_Dataset = gtk_plot_dataset_new 
		( GTK_PLOT( Measure_Graph) );
	gtk_plot_add_dataset( GTK_PLOT( Measure_Graph ),
		Measure_U_Dataset );
	gtk_plot_dataset_set_line_attributes( Measure_U_Dataset,
		GTK_PLOT_LINE_SOLID, 2, u_color );
	gtk_plot_dataset_set_connector( Measure_U_Dataset,
		GTK_PLOT_CONNECT_HV_STEP );
	gtk_plot_dataset_set_symbol( Measure_U_Dataset,
    	GTK_PLOT_SYMBOL_CIRCLE,
    	GTK_PLOT_SYMBOL_OPAQUE,
    	4, 1, point_color );
    gtk_plot_dataset_set_legend( Measure_U_Dataset,
        MEASURE_U_TEXT );
	
	gdk_color_parse( "white", &measure_color );
	gdk_color_alloc( gtk_widget_get_colormap( Measure_Layout ),
		&measure_color );
	gtk_plot_legends_set_attributes( GTK_PLOT( Measure_Graph ),
        NULL, 0, NULL, &measure_color );
	}  



/************************************************************************
 *  function which is call to refresh the value of window               *
 ************************************************************************/


void measure_show( void )
	{
    gfloat x,dx;
    
    x = N*TS/1000.0;
    dx = (x/10.0);    
    
	gtk_widget_show_all( Measure_Window );
	gtk_plot_set_range( GTK_PLOT( Measure_Graph ),
		0, x, -10, 10 );
	gtk_plot_axis_set_ticks( GTK_PLOT( Measure_Graph ), 
		GTK_PLOT_AXIS_LEFT, 2, 1 );
	gtk_plot_axis_set_ticks( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_BOTTOM, 2*dx, dx );
	gtk_plot_axis_set_ticks( GTK_PLOT( Measure_Graph ), 
		GTK_PLOT_AXIS_RIGHT, 2, 1 );
	gtk_plot_axis_set_ticks( GTK_PLOT( Measure_Graph ),
		GTK_PLOT_AXIS_TOP, 2*dx, dx );
	gtk_plot_dataset_set_points( Measure_Y_Dataset, y, N );
	gtk_plot_dataset_set_points( Measure_U_Dataset, u, N );
	gtk_widget_draw( Measure_Layout,NULL );
	}


/************************************************************************
 *  functions called by the other files                                 *
 ************************************************************************/

int measure_init( void )
	{
	gdk_color_parse( "black", &point_color );
	gdk_color_parse( "blue", &y_color );
	gdk_color_parse( "red", &u_color );

	gdk_color_alloc( gdk_colormap_get_system( ), &point_color );
	gdk_color_alloc( gdk_colormap_get_system( ), &y_color );
	gdk_color_alloc( gdk_colormap_get_system( ), &u_color );

	if ( Measure_Window == NULL ) 
		measure_window_create();
 	gtk_signal_connect( GTK_OBJECT( Measure_Window ),
		"delete_event",
		GTK_SIGNAL_FUNC( measure_window_close ), 
		NULL );
	gtk_widget_show_all( Measure_Window );
            
	return 0;
	}  

