/************************************************************************
 *                                                                      *
 *  print.c  ---   This file prints the stuff.                          *
 *                                                                      *
 *  This file is part of the control/rt_control/xcontrol package which  *
 *  is free software; you can redistribute it and/or modify it under    *
 *  the terms of the version 2 of GNU General Public License as         *
 *  published by the Free Software Foundation.                          *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft 1999                                                       *
 *  Peter Wurmsdobler                                                   *
 *                                                                      *
 ************************************************************************/

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gtkplot.h>
#include <gtkplotlayout.h>
#include "common.h"
#include "print.h"
#include "dialog.h"

/*
 *	Debug macros for 
 */
//#define PRINT_DEBUG
#ifdef PRINT_DEBUG
	#define DPRINTF0( message )		printf( message )
	#define DPRINTF1( message,a )	printf( message,a )
	#define DPRINTF2( message,a,b )	printf( message,a,b )
#else
	#define DPRINTF0( message )
	#define DPRINTF1( message,a )
	#define DPRINTF2( message,a,b )
#endif

#define PRINT_FILENAME			( "/tmp/Xmcm.ps" )
#define MAX_FILENAME_LENGTH		( 256 )
#define MAX_PRINTCOMMAND_LENGTH	( 256 )


int print_lock = 0;
int print_file_lock = 0;

/************************************************************************
 *  definition of widgets for torque/speed                              *
 ************************************************************************/

/*
 *	definition of widgets for torque/speed print dialog
 */
GtkWidget *Print_Dialog_Window;
GtkWidget *Print_Container;
GSList    *Print_Container_Group = NULL;
GtkWidget *Print_Printer_Radiobutton;
GtkWidget *Print_File_Radiobutton;
GtkWidget *Print_File_Name_Entry;
GtkWidget *Print_Command_Entry;
GtkWidget *Print_Browse_Button;
GtkWidget *Print_OK_Button;
GtkWidget *Print_Cancel_Button;
GtkWidget *Print_File_Name_Label;
GtkWidget *Print_Command_Label;
GtkWidget *Print_Fileselection;
GtkPlot   *Print_Actual_Plot;


/************************************************************************
 *  Callback function for torque/speed measure printing                 *
 ************************************************************************/

void print_dialog_window_hide( void )
	{
	DPRINTF0( "Entering print_dialog_window_hide\n" );
	gtk_widget_hide( Print_Dialog_Window );
    print_lock = 0;
	DPRINTF0( "Leaving print_dialog_window_hide\n" );
	}

void print_dialog_ok_button_clicked( void )
	{
	gboolean print_selection;
	gchar print_command[MAX_PRINTCOMMAND_LENGTH];
//	gchar print_filename[MAX_FILENAME_LENGTH];
		
	DPRINTF0( "Entering print_dialog_ok_button_clicked\n" );

	print_selection = gtk_toggle_button_get_active( 
    	GTK_TOGGLE_BUTTON( Print_Printer_Radiobutton )  );
		
	switch( print_selection )
		{
		case TRUE: 
 			gtk_plot_export_ps( 
				GTK_PLOT( Print_Actual_Plot ), PRINT_FILENAME, 
                GTK_PLOT_LANDSCAPE, 0, GTK_PLOT_A4 );
            strcpy( print_command, gtk_entry_get_text( 
            	GTK_ENTRY( Print_Command_Entry ) ));
			strcat( print_command, " " );			
			strcat( print_command, PRINT_FILENAME );			
			DPRINTF1( "print_command = %s \n", print_command );
            system( print_command );
			break;
            			
		case FALSE:
			gtk_plot_export_ps( 
            	GTK_PLOT( Print_Actual_Plot ), 
                gtk_entry_get_text( GTK_ENTRY( Print_File_Name_Entry )  ), 
                GTK_PLOT_LANDSCAPE, 0, GTK_PLOT_A4  );
			break;

		default:
			/* we should not arrive here */;
		}

	print_dialog_window_hide();
    print_lock = 0;

	DPRINTF0( "Leaving print_dialog_ok_button_clicked\n" );
	}

void print_file_radiobutton_clicked( void )
	{
	gtk_widget_set_sensitive( Print_File_Name_Label, TRUE );
	gtk_widget_set_sensitive( Print_Browse_Button, TRUE );
	gtk_widget_set_sensitive( Print_File_Name_Entry, TRUE );

	gtk_widget_set_sensitive( Print_Command_Entry, FALSE );
	gtk_widget_set_sensitive( Print_Command_Label, FALSE );
	}


void print_printer_radiobutton_clicked( void )
	{
	gtk_widget_set_sensitive( Print_Command_Entry, TRUE );
	gtk_widget_set_sensitive( Print_Command_Label, TRUE );

	gtk_widget_set_sensitive( Print_File_Name_Label, FALSE );
	gtk_widget_set_sensitive( Print_Browse_Button, FALSE );
	gtk_widget_set_sensitive( Print_File_Name_Entry, FALSE );
	
	}


void print_fileselection_ok_button_clicked( void )
	{
	gchar *print_file_name;
		
	print_file_name = gtk_file_selection_get_filename 
		( GTK_FILE_SELECTION( Print_Fileselection ) );
	
	gtk_entry_set_text( GTK_ENTRY
		( Print_File_Name_Entry ), print_file_name );
	
	gtk_widget_hide( Print_Fileselection );
	}


void print_fileselection_cancel_button_clicked( void )
	{
	gtk_widget_hide( Print_Fileselection );
	}


/************************************************************************
 *  Defintion of torque speed file selection                            *
 ************************************************************************/

void print_fileselection_create( void )
	{
   	Print_Fileselection = 
  		gtk_file_selection_new( FILE_SELECTION_TEXT );
  	gtk_container_border_width( GTK_CONTAINER
  		( Print_Fileselection ), 10 );

  	GTK_WIDGET_SET_FLAGS( ( GTK_FILE_SELECTION 
  		( Print_Fileselection ) )->ok_button,
  		GTK_CAN_DEFAULT );
	gtk_signal_connect( GTK_OBJECT( ( GTK_FILE_SELECTION
		( Print_Fileselection ) )->ok_button ),
		"clicked",
		GTK_SIGNAL_FUNC( print_fileselection_ok_button_clicked ),
		NULL );

	
  	GTK_WIDGET_SET_FLAGS( ( GTK_FILE_SELECTION
  		( Print_Fileselection ) )->cancel_button,GTK_CAN_DEFAULT );
	gtk_signal_connect( GTK_OBJECT( ( GTK_FILE_SELECTION
		( Print_Fileselection ) )->cancel_button ),
		"clicked",
		GTK_SIGNAL_FUNC( print_fileselection_cancel_button_clicked ),
		NULL );
	}


void print_browse_button_clicked( void )
	{	
	/* if the window does not exist, create it 
	 */
	if( Print_Fileselection == NULL ) 
		print_fileselection_create( );

	/* if there is already a file selection window active,
	 */	
	if ( print_file_lock < 0 )
		dialog_error_window( FILE_SELECTION_WINDOW_OCCUPIED_TEXT,
        	sizeof(FILE_SELECTION_WINDOW_OCCUPIED_TEXT) );
	else
        print_file_lock = -1;

	/* show acitve plot dialog 
	 */	
	gtk_widget_show( Print_Fileselection );
	}


/************************************************************************
 *  definition of print dialog window for torque speed measure          *
 ************************************************************************/

void print_dialog_window_create( void )
	{
	Print_Dialog_Window = 
		gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title( GTK_WINDOW 
		( Print_Dialog_Window ), PRINT_TEXT );
	gtk_widget_set_usize( Print_Dialog_Window, 
		295, 200 );
	gtk_widget_show( Print_Dialog_Window );

	gtk_signal_connect( GTK_OBJECT( Print_Dialog_Window ),
		"delete_event",
		GTK_SIGNAL_FUNC( print_dialog_window_hide ), 
		NULL );		

  	Print_Container = gtk_fixed_new(  );
  	gtk_widget_show( Print_Container );
  	gtk_container_add( GTK_CONTAINER 
  		( Print_Dialog_Window ), 
  		Print_Container );
  	gtk_widget_set_usize( Print_Container, 
  		150, 200 );

  	Print_Printer_Radiobutton = 
    	gtk_radio_button_new_with_label( Print_Container_Group, TO_PRINTER_TEXT );
  	Print_Container_Group = gtk_radio_button_group( 
    	GTK_RADIO_BUTTON( Print_Printer_Radiobutton )  );
  	gtk_widget_show( Print_Printer_Radiobutton );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_Printer_Radiobutton, 32, 5 );
  	gtk_widget_set_usize( Print_Printer_Radiobutton, 93, 24 );
  	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON 
  		( Print_Printer_Radiobutton ), TRUE );

  	gtk_signal_connect( GTK_OBJECT( Print_Printer_Radiobutton ),
		"clicked",
		GTK_SIGNAL_FUNC( print_printer_radiobutton_clicked ), 
 		NULL );		

  	Print_File_Radiobutton = gtk_radio_button_new_with_label( Print_Container_Group, TO_FILE_TEXT );
  	Print_Container_Group = gtk_radio_button_group( GTK_RADIO_BUTTON( Print_File_Radiobutton ) );
  	gtk_widget_show( Print_File_Radiobutton );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_File_Radiobutton, 220, 5 );
  	gtk_widget_set_usize( Print_File_Radiobutton, 93, 24 );
  
  	gtk_signal_connect( GTK_OBJECT( Print_File_Radiobutton ),
		"clicked",
		GTK_SIGNAL_FUNC( print_file_radiobutton_clicked ), 
		NULL );		

  	Print_File_Name_Entry = gtk_entry_new(  );
  	gtk_entry_set_text( GTK_ENTRY
		( Print_File_Name_Entry ),FILE_EXT_TEXT );
  	gtk_widget_show( Print_File_Name_Entry );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_File_Name_Entry, 32, 108 );
  	gtk_widget_set_usize( Print_File_Name_Entry, 200, 22 );
  	gtk_widget_set_sensitive( Print_File_Name_Entry,FALSE );

  	Print_Command_Entry = gtk_entry_new(  );
  	gtk_entry_set_text( GTK_ENTRY
		( Print_Command_Entry ),COMMAND_TEXT );
  	gtk_widget_show( Print_Command_Entry );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_Command_Entry, 32, 60 );
  	gtk_widget_set_usize( Print_Command_Entry, 158, 22 );

  	Print_Browse_Button = gtk_button_new_with_label( BROWSE_TEXT );
  	gtk_widget_show( Print_Browse_Button );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_Browse_Button, 234, 108 );
  	gtk_widget_set_usize( Print_Browse_Button, 47, 22 );
  	gtk_widget_set_sensitive( Print_Browse_Button,FALSE );
  
  	gtk_signal_connect( GTK_OBJECT( Print_Browse_Button ),
		"clicked",
		GTK_SIGNAL_FUNC( print_browse_button_clicked ), 
 		NULL );		

  	Print_OK_Button = gtk_button_new_with_label( OK_BUTTON_TEXT );
  	gtk_widget_show( Print_OK_Button );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_OK_Button, 24, 172 );
  	gtk_widget_set_usize( Print_OK_Button, 47, 22 );
 
  	/*Print_Printer_Radiobuttstate=gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON 
  		( Print_Printer_Radiobutton ) );
  	g_print( "Print_Printer_Radiobuttstate = %d\n",Print_Printer_Radiobuttstate );*/
  
  	gtk_signal_connect( GTK_OBJECT( Print_OK_Button ),
		"clicked",
		GTK_SIGNAL_FUNC( print_dialog_ok_button_clicked ), 
		NULL );		

  	Print_Cancel_Button = gtk_button_new_with_label( CANCEL_BUTTON_TEXT );
  	gtk_widget_show( Print_Cancel_Button );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_Cancel_Button, 224, 172 );
  	gtk_widget_set_usize( Print_Cancel_Button, 47, 22 );
  
  	gtk_signal_connect( GTK_OBJECT( Print_Cancel_Button ),
		"clicked",
		GTK_SIGNAL_FUNC( print_dialog_window_hide ), 
		NULL );		

  	Print_File_Name_Label = gtk_label_new( FILE_NAME_TEXT );
  	gtk_widget_show( Print_File_Name_Label );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_File_Name_Label, 22, 92 );
  	gtk_widget_set_usize( Print_File_Name_Label, 77, 16 );
  	gtk_widget_set_sensitive( Print_File_Name_Label,FALSE );

  	Print_Command_Label = gtk_label_new( PRINT_COMMAND_TEXT );
  	gtk_widget_show( Print_Command_Label );
  	gtk_label_set_justify( GTK_LABEL( Print_Command_Label ), GTK_JUSTIFY_LEFT );
  	gtk_fixed_put( GTK_FIXED( Print_Container ), Print_Command_Label, 15, 44 );
  	gtk_widget_set_usize( Print_Command_Label, 127, 16 );
	
	printf( "Leaving create_print_dialog_window\n" );	
	}

    
/*
 *	Print GtkPlot Widget passed
 */
void print_dialog_window( GtkPlot *plot )
	{
	/* if the window does not exist, create it 
	 */
	if ( Print_Dialog_Window == NULL ) 
		print_dialog_window_create();

	/* if there is already a print window active,
	 */	
	if ( print_lock < 0 )
		dialog_error_window( PRINT_WINDOW_OCCUPIED_TEXT,
        	sizeof(PRINT_WINDOW_OCCUPIED_TEXT) );
	else
    	{
        print_lock = -1;
	  	Print_Actual_Plot = plot;
		}

	/* show acitve plot dialog 
	 */	
	gtk_widget_show_all( Print_Dialog_Window );
  	}
