/************************************************************************
 *                                                                      *
 *  rt_bmc1000-params.h  ---  parameter header file for rt_bmc1000.c.   *
 *                                                                      *
 *  This file declares common variables and constants for the BMC1000   *
 *  DAQ board used in the board module rt_bmc1000.c                     *
 *                                                                      *
 *  This file is part of the control/rt_control/xcontrol package which  *
 *  is free software; you can redistribute it and/or modify it under    *
 *  the terms of the version 2 of GNU General Public License as         *
 *  published by the Free Software Foundation.                          *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft 1999                                                       *
 *  Peter Wurmsdobler                                                   *
 *                                                                      *
 ************************************************************************/

/************************************************************************
 *                                                                      * 
 *  Settings on the DAQ board which the following                       *
 *  driver definitions are valid for:                                   *
 *                                                                      *
 *      JP1    C       TRIG0 connected to DI0                           *
 *      JP2    NC      GATE0 not connected                              * 
 *      JP3    INT     internal voltage reference                       *
 *      JP4    INT     internal voltage reference                       *
 *      JP5    -5V     DAC voltage reference                            *
 *      JP6    3       default                                          *
 *                                                                      * 
 ************************************************************************/

#define RT_BMC1000_BASE_ADDRESS		    (0x230)	/*	i/o base address  */
#define RT_BMC1000_ADDRESS_RANGE		(16)	/*	i/o address range */


/*
 *	value for multiplexer and range selection
 */
#define RT_BMC1000_MUX_ADDRESS			(RT_BMC1000_BASE_ADDRESS + 4)


/*
 *	values for ADC used in driver
 */
#define RT_BMC1000_ADC_CHANNELS			(16)
#define RT_BMC1000_ADCL_ADDRESS			(RT_BMC1000_BASE_ADDRESS + 8)
#define RT_BMC1000_ADCH_ADDRESS			(RT_BMC1000_BASE_ADDRESS + 9)
/*
 * (11 10 9 8 7 6 5 4) = high byte
 * ( 3  2 1 0 x x x x) = low byte 
 */
#define RT_BMC1000_ADC_SHIFT			(4)
#define RT_BMC1000_ADC_START_ADDRESS	(RT_BMC1000_BASE_ADDRESS + 8)


/*
 *	values for DAC used in driver
 */
#define RT_BMC1000_DAC_CHANNELS			(2)
#define RT_BMC1000_DAC_ADDRESSES		{{(RT_BMC1000_BASE_ADDRESS + 0x0c ) ,	\
										  (RT_BMC1000_BASE_ADDRESS + 0x0d )},	\
										 {(RT_BMC1000_BASE_ADDRESS + 0x0e ) ,	\
										  (RT_BMC1000_BASE_ADDRESS + 0x0f )}}
/*
 * (11 10 9 8 7 6 5 4) = high byte
 * ( 3  2 1 0 x x x x) = low byte 
 */

/*
 *	values for DO
 */
#define RT_BMC1000_DIO_BYTES			(4)
#define RT_BMC1000_DIO_ADDRESSES		{{(RT_BMC1000_BASE_ADDRESS + 3)  ,	\
										  (RT_BMC1000_BASE_ADDRESS + 11)},	\
                                    	 {(RT_BMC1000_BASE_ADDRESS + 3)  ,	\
										  (RT_BMC1000_BASE_ADDRESS + 11)}}
