void rt_control_task( int t )
    {
    unsigned int l,h;

    RT_FILTER_DPRINTF0( "Starting rt_control_task\n" );

    /* get the value from the board at channel preset in init_module
     */
    Y = rt_bmc1000_aget();    

    /* control deviation, BIAS is cancelled out if setpoint
     * is {0,65535}.
     */
    e[k] = (int) ( W - Y ); /* this is the control deviation ! */

    /* THE control algorithm, a simple IIR filter
     */
    u[k] = (shm->b[0]) * e[k];
    for ( l=1; l<LENGTH; l++ )
        {
        h = ( k - l + LENGTH ) % LENGTH;
        u[k] += ( (shm->b[l]) * e[h] - (shm->a[l]) * u[h] );
        }
    u[k] /= SCALE;   /* this is because control parameters are scaled */
    k = (k+1) % LENGTH;    

    /* control variable output, BIAS is added to shift to {0,65535}.
     */
    U = (unsigned short int)( u[k] + BIAS );
    
    /* set the control current on board and the loop is closed
     */
    rt_bmc1000_aset( OUTPUT_CHANNEL, U );

    /* if a measurement of step response is set, 
     */
    if (measure == 0)
        {
    	/* we store the values in shm
    	 */
        shm->y[index] = Y;
        shm->u[index] = U;
        
		/* 
		 * increase index and stop measurment of step response
     	 */
        index++;
        if (index == (shm->length) )
        	{
            rt_control_event_msg( MEASURE_READY );
            measure = -1;
        	}
        } 

    Y = rt_bmc1000_aget();
    printk("Y = %d\n",Y);
    U = Y;    
    rt_bmc1000_aset( OUTPUT_CHANNEL, U );
        
    RT_FILTER_DPRINTF0( "Leaving rt_control_task\n" );
    rt_task_wait(); /* scheduler will take over now */
    }


/*
 *    rt_control_start -- start control
 */
static void rt_control_start(void)
    { 
    RTIME period;
    struct timespec T_ts;
    long long T_ns;
    
    RT_FILTER_DPRINTF0( "Entering rt_control_start\n" );

	/* cleaning the buffers and reset k  
	 */
    for (k=0; k<LENGTH; k++)
    	{
        e[k] = 0;
        u[k] = 0;
        }
    k = 0;
    
	/* start the control process by setting task periodic at PERIOD
	 */ 
    T_ns = TS*1000; /* convert to ns */
    T_ts = timespec_from_ns( T_ns );
    period = timespec_to_RTIME( T_ts );
//    rt_task_make_periodic( &control, rt_get_time()+3000, period );

    rt_task_wakeup( &control );
   
    RT_FILTER_DPRINTF0( "Leaving rt_control_start\n" );
    }
