/************************************************************************
 *                                                                      *
 *  xcontrol.c  ---  the xcontrol main window and menus.                *
 *                                                                      *
 *  This file is part of the control/rt_control/xcontrol package which  *
 *  is free software; you can redistribute it and/or modify it under    *
 *  the terms of the version 2 of GNU General Public License as         *
 *  published by the Free Software Foundation.                          *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft 1999                                                       *
 *  Peter Wurmsdobler                                                   *
 *                                                                      *
 ************************************************************************/

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gtkplot.h>
#include "common.h"
#include "control.h"
#include "xcontrol.h"
#include "misc.h"
#include "measure.h"
#include "dialog.h"
 
/*
 *	Debug macros for 
 */
//#define XCONTROL_DEBUG
#ifdef XCONTROL_DEBUG
	#define DPRINTF0(message)		printf(message)
	#define DPRINTF1(message,a)		printf(message,a)
	#define DPRINTF2(message,a,b)	printf(message,a,b)
#else
	#define DPRINTF0(message)
	#define DPRINTF1(message,a)
	#define DPRINTF2(message,a,b)
#endif


/*
 *	definition of control parameters
 */
gfloat w;			/* set point */
gint N;				/* number of samples samples */
gfloat a[LENGTH];	/* denominator of dicrete transfer function */
gfloat b[LENGTH];	/* numberator of dicrete transfer function */


/************************************************************************
 *  definition of widgets for main window                               *
 ************************************************************************/

/*
 *	definition of widgets for main window
 */
GtkWidget *Xcontrol_Window;
GtkWidget *Xcontrol_Window_Container;
GtkWidget *Xcontrol_Window_Start_Button;
GtkWidget *Xcontrol_Window_Stop_Button;
GtkWidget *Xcontrol_Window_W_Text;
GtkWidget *Xcontrol_Window_N_Text;
GtkWidget *Xcontrol_Window_W_Entry;
GtkWidget *Xcontrol_Window_N_Entry;
GtkWidget *Xcontrol_Window_a_Entry[LENGTH];
GtkWidget *Xcontrol_Window_b_Entry[LENGTH];
GtkWidget *Xcontrol_Window_a_Text[LENGTH];
GtkWidget *Xcontrol_Window_b_Text[LENGTH];
GtkWidget *Xcontrol_Window_Gd1_Text;
GtkWidget *Xcontrol_Window_Gd2_Text;
GtkWidget *Xcontrol_Window_Gd3_Text;
GtkWidget *Xcontrol_Window_Toolbar;
 
GtkStyle  *StartButtonStyle;
GtkStyle  *StopButtonStyle;
GdkColor   ButtonNormalColor;

 
/*
 *	definition of widgets for menu which is contain in main window       *
 */ 
GtkWidget *Xcontrol_Window_Menu;
GtkWidget *Xcontrol_Window_File;
GtkWidget *Xcontrol_Window_Menu_File;
GtkWidget *Xcontrol_Window_Quit_Menu;
GSList    *Xcontrol_Window_Mode_Group = NULL;

gchar *measure_x_title;


/************************************************************************
 *  Callback functions for window                                       *
 ************************************************************************/


void xcontrol_measure_start_menu_activate( void )
	{		
	gtk_widget_set_sensitive( Xcontrol_Window_Start_Button, FALSE );
	gtk_widget_set_sensitive( Xcontrol_Window_Stop_Button, TRUE );
	gtk_widget_set_sensitive( Xcontrol_Window_Menu, FALSE );

	control_start();
	}


void xcontrol_measure_stop_menu_activate( void )
	{
	gtk_widget_show_all( Xcontrol_Window );
	gtk_widget_set_sensitive( Xcontrol_Window_Start_Button, TRUE );
	gtk_widget_set_sensitive( Xcontrol_Window_Stop_Button, FALSE );
	gtk_widget_set_sensitive( Xcontrol_Window_Menu, TRUE );
		
	control_stop();
	}

  
void xcontrol_w_entry_callback( GtkWidget *entry )
	{
	w = gchar2gfloat( gtk_entry_get_text( GTK_ENTRY( entry ) ) );	
	control_set_w();
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_W_Entry ),
    	gfloat2gchar( w, 3 ) );
    }



void xcontrol_N_entry_callback( GtkWidget *entry )
	{
	N = gchar2gfloat( gtk_entry_get_text( GTK_ENTRY( entry ) ) );	
	control_set_N();
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_N_Entry ),
    	gint2gchar( N ) );
	}

void xcontrol_a_entry_callback( GtkWidget *entry )
	{
    int k;
    
    for (k=0; k<LENGTH; k++)
    	if (entry == Xcontrol_Window_a_Entry[k])
        	break;
            
	a[k] = gchar2gfloat( gtk_entry_get_text( GTK_ENTRY( entry ) ) );	
	if (k == 0)
    	a[k] = 1; /* this should be 1 */
    control_set_ab();
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_a_Entry[k] ),
    	gfloat2gchar( a[k], 5 ) );
	}

void xcontrol_b_entry_callback( GtkWidget *entry )
	{
    int k;
    
    for (k=0; k<LENGTH; k++)
    	if (entry == Xcontrol_Window_b_Entry[k])
        	break;
            
	b[k] = gchar2gfloat( gtk_entry_get_text( GTK_ENTRY( entry ) ) );	
    control_set_ab();
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_b_Entry[k] ),
    	gfloat2gchar( b[k], 5 ) );
	}


/************************************************************************
 *  Callback functions of window menus                                  *
 ************************************************************************/


void xcontrol_quit_menu_activate( void )
	{
	gtk_main_quit();
	}



/************************************************************************
 * definition of main window                                            *
 ************************************************************************/

void xcontrol_window_create( void )
	{
    gchar ab[20];
    gint k;

 	Xcontrol_Window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  	gtk_widget_set_usize( Xcontrol_Window, 390, 300 );
  	gtk_window_set_title( GTK_WINDOW( Xcontrol_Window ), MAIN_WINDOW_TEXT );
  	gtk_window_set_policy( GTK_WINDOW( Xcontrol_Window ), FALSE, FALSE, FALSE );
  	
  	Xcontrol_Window_Container = gtk_fixed_new();
  	gtk_container_add( GTK_CONTAINER( Xcontrol_Window ), Xcontrol_Window_Container );

  	Xcontrol_Window_Toolbar = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL,
  		GTK_TOOLBAR_TEXT );
	gtk_widget_set_usize( Xcontrol_Window_Toolbar, 390, 50 );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),Xcontrol_Window_Toolbar, 0, 20 );
	gtk_container_set_border_width( GTK_CONTAINER( Xcontrol_Window_Toolbar ) , 5 );
    gtk_toolbar_set_space_size( GTK_TOOLBAR( Xcontrol_Window_Toolbar ), 5 );
		
	Xcontrol_Window_Start_Button = gtk_toolbar_append_item( GTK_TOOLBAR 
		( Xcontrol_Window_Toolbar ), 
		MEASURE_START_TEXT, MEASURE_START_TOOLTIPS_TEXT, "Private", NULL,
    	GTK_SIGNAL_FUNC( xcontrol_measure_start_menu_activate ), NULL );
	gtk_widget_set_usize( Xcontrol_Window_Start_Button, 190, 20 );
	
	Xcontrol_Window_Stop_Button = gtk_toolbar_append_item( GTK_TOOLBAR 
		( Xcontrol_Window_Toolbar ), 
		MEASURE_STOP_TEXT, MEASURE_STOP_TOOLTIPS_TEXT, "Private",NULL,
        GTK_SIGNAL_FUNC( xcontrol_measure_stop_menu_activate ),NULL );
	gtk_widget_set_usize( Xcontrol_Window_Stop_Button, 190, 20 );
	gtk_widget_set_sensitive( Xcontrol_Window_Stop_Button, FALSE );
	
	
	Xcontrol_Window_W_Entry = gtk_entry_new_with_max_length( 25 );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_W_Entry, 210, 70 );
	gtk_widget_set_usize( Xcontrol_Window_W_Entry, 100, 25 );
	gtk_signal_connect( GTK_OBJECT( Xcontrol_Window_W_Entry ),
		"activate",
		GTK_SIGNAL_FUNC( xcontrol_w_entry_callback ),
		Xcontrol_Window_W_Entry );
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_W_Entry ),
		gfloat2gchar( w, 3 ) );
       
    Xcontrol_Window_N_Entry = gtk_entry_new_with_max_length( 25 );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_N_Entry, 210, 100 );
	gtk_widget_set_usize( Xcontrol_Window_N_Entry, 100, 25 );
	gtk_signal_connect( GTK_OBJECT( Xcontrol_Window_N_Entry ),
		"activate",
		GTK_SIGNAL_FUNC( xcontrol_N_entry_callback ),
		Xcontrol_Window_N_Entry );
	gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_N_Entry ),
		gint2gchar( N ) );
	
  	Xcontrol_Window_W_Text = gtk_label_new( XCONTROL_W_TEXT );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_W_Text, 45, 70 );
	gtk_widget_set_usize( Xcontrol_Window_W_Text, 160, 22 );
 	gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_W_Text ),
 		GTK_JUSTIFY_RIGHT );

	Xcontrol_Window_N_Text = gtk_label_new( XCONTROL_N_TEXT );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_N_Text, 45, 100 );
	gtk_widget_set_usize( Xcontrol_Window_N_Text, 160, 25 );
	gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_N_Text ),
		GTK_JUSTIFY_RIGHT );

	Xcontrol_Window_Gd1_Text = gtk_label_new( XCONTROL_GD1_TEXT );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_Gd1_Text, 45, 130 );
	gtk_widget_set_usize( Xcontrol_Window_Gd1_Text, 300, 25 );
	gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_Gd1_Text ),
		GTK_JUSTIFY_RIGHT );

	Xcontrol_Window_Gd2_Text = gtk_label_new( XCONTROL_GD2_TEXT );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_Gd2_Text, 45, 145 );
	gtk_widget_set_usize( Xcontrol_Window_Gd2_Text, 300, 25 );
	gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_Gd2_Text ),
		GTK_JUSTIFY_RIGHT );

	Xcontrol_Window_Gd3_Text = gtk_label_new( XCONTROL_GD3_TEXT );
	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
		Xcontrol_Window_Gd3_Text, 45, 160 );
	gtk_widget_set_usize( Xcontrol_Window_Gd3_Text, 300, 25 );
	gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_Gd3_Text ),
		GTK_JUSTIFY_RIGHT );

	for ( k=0; k<LENGTH; k++)
    	{
        strcpy( ab, "b[" );  
        strcat( ab, gint2gchar( k ) );
		strcat( ab, "] = " );
        
		Xcontrol_Window_b_Text[k] = gtk_label_new( ab );
		gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
			Xcontrol_Window_b_Text[k], 50, 200+(k*30) );
		gtk_widget_set_usize( Xcontrol_Window_b_Text[k], 50, 25 );
		gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_b_Text[k] ),
			GTK_JUSTIFY_RIGHT );

		Xcontrol_Window_b_Entry[k] = gtk_entry_new_with_max_length( 5 );
		gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
			Xcontrol_Window_b_Entry[k], 100, 200+(k*30) );
		gtk_widget_set_usize( Xcontrol_Window_b_Entry[k], 50, 25 );
		gtk_signal_connect( GTK_OBJECT( Xcontrol_Window_b_Entry[k] ),
			"activate", GTK_SIGNAL_FUNC( xcontrol_b_entry_callback ),
			Xcontrol_Window_b_Entry[k] );
		gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_b_Entry[k] ),
			gfloat2gchar( b[k], 3 ) );

        strcpy( ab, "a[" );  
		strcat( ab, gint2gchar( k ) );
		strcat( ab, "] = " );
        
		Xcontrol_Window_a_Text[k] = gtk_label_new( ab );
		gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
			Xcontrol_Window_a_Text[k], 200, 200+(k*30) );
		gtk_widget_set_usize( Xcontrol_Window_a_Text[k], 50, 25 );
		gtk_label_set_justify( GTK_LABEL( Xcontrol_Window_a_Text[k] ),
			GTK_JUSTIFY_RIGHT );

		Xcontrol_Window_a_Entry[k] = gtk_entry_new_with_max_length( 5 );
		gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),
			Xcontrol_Window_a_Entry[k], 250, 200+(k*30) );
		gtk_widget_set_usize( Xcontrol_Window_a_Entry[k], 50, 25 );
		gtk_signal_connect( GTK_OBJECT( Xcontrol_Window_a_Entry[k] ),
			"activate",
			GTK_SIGNAL_FUNC( xcontrol_a_entry_callback ),
			Xcontrol_Window_a_Entry[k] );
		gtk_entry_set_text( GTK_ENTRY( Xcontrol_Window_a_Entry[k] ),
			gfloat2gchar( a[k], 3 ) );
        }

	}



/************************************************************************
 *  definition of menu and menu items                                   *
 ************************************************************************/


void xcontrol_menu_create( void )
	{
	Xcontrol_Window_Menu_File = gtk_menu_new();
  	  		
	Xcontrol_Window_Quit_Menu = gtk_menu_item_new_with_label( QUIT_TEXT );
  	gtk_menu_append( GTK_MENU( Xcontrol_Window_Menu_File ), Xcontrol_Window_Quit_Menu );
  	gtk_signal_connect( GTK_OBJECT( Xcontrol_Window_Quit_Menu ), 
  		"activate",
    	GTK_SIGNAL_FUNC( xcontrol_quit_menu_activate ),
    	NULL );
  
	Xcontrol_Window_File = gtk_menu_item_new_with_label( FILE_TEXT );
  	gtk_menu_item_set_submenu( GTK_MENU_ITEM( Xcontrol_Window_File ),
  		Xcontrol_Window_Menu_File );
	
	Xcontrol_Window_Menu = gtk_menu_bar_new();
  	gtk_menu_bar_append( GTK_MENU_BAR( Xcontrol_Window_Menu ), Xcontrol_Window_File );
 	gtk_fixed_put( GTK_FIXED( Xcontrol_Window_Container ),Xcontrol_Window_Menu , 0,0 );
  	gtk_widget_set_usize( Xcontrol_Window_Menu, 390, 32 );
  	
	}
	

/************************************************************************
 * functions called by other files                                      *
 ************************************************************************/

int xcontrol_init( void )
	{
    int l;
    
    
	/* init the values
     */
    w = 0.0;
    N = 10;
    a[0] = 1.0;
    b[0] = 1.0;
    for (l=1; l<LENGTH; l++)
        {
        a[l] = 0.0;
        b[l] = 0.0;
        }    
	
	/* test if the main window exist
     */
	if ( Xcontrol_Window == NULL )
		xcontrol_window_create();
	gtk_widget_show_all( Xcontrol_Window );
	
	if ( Xcontrol_Window_Menu == NULL )
		xcontrol_menu_create();
	gtk_widget_show_all( Xcontrol_Window_Menu );
	
    /* if a "destroy window" occured : 
     */
	gtk_signal_connect( GTK_OBJECT( Xcontrol_Window ), 
		"delete_event",
		GTK_SIGNAL_FUNC( xcontrol_quit_menu_activate ), 
		NULL );		

  	return 0;
	}	
